/*
 * This file is part of architectury.
 * Copyright (C) 2020, 2021 architectury
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package me.shedaniel.architectury.registry.fabric;

import me.shedaniel.architectury.registry.ParticleFactories;
import net.fabricmc.fabric.api.client.particle.v1.FabricSpriteProvider;
import net.fabricmc.fabric.api.client.particle.v1.ParticleFactoryRegistry;
import net.minecraft.client.particle.ParticleProvider;
import net.minecraft.client.renderer.texture.TextureAtlas;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.ParticleType;

import java.util.List;
import java.util.Random;

public class ParticleFactoriesImpl {
    public static class ExtendedSpriteSetImpl implements ParticleFactories.ExtendedSpriteSet {
        private final FabricSpriteProvider delegate;
    
        public ExtendedSpriteSetImpl(FabricSpriteProvider delegate) {
            this.delegate = delegate;
        }
    
        @Override
        public TextureAtlas getAtlas() {
            return delegate.getAtlas();
        }
    
        @Override
        public List<TextureAtlasSprite> getSprites() {
            return delegate.getSprites();
        }
    
        @Override
        public TextureAtlasSprite get(int i, int j) {
            return delegate.get(i, j);
        }
    
        @Override
        public TextureAtlasSprite get(Random random) {
            return delegate.get(random);
        }
    }

    public static <T extends ParticleOptions> void register(ParticleType<T> type, ParticleProvider<T> provider) {
        ParticleFactoryRegistry.getInstance().register(type, provider);
    }

    public static <T extends ParticleOptions> void register(ParticleType<T> type, ParticleFactories.PendingParticleProvider<T> constructor) {
        ParticleFactoryRegistry.getInstance().register(type, provider -> constructor.create(new ExtendedSpriteSetImpl(provider)));
    }
}
